@@ -3,7 +3,7 @@ package Alien::Base::ModuleBuild;
 use strict;
 use warnings;
 
-our $VERSION = '0.000_021';
+our $VERSION = '0.000_022';
 $VERSION = eval $VERSION;
 
 use parent 'Module::Build';
@@ -54,7 +54,7 @@ __PACKAGE__->add_property( alien_temp_dir => '_alien' );
 # alien_share_dir: folder name for the "install" of the library
 # this is added (unshifted) to the @{share_dir->{dist}}  
 # N.B. is reset during constructor to be full folder name 
-__PACKAGE__->add_property('alien_share_dir'); # default => '_install'
+__PACKAGE__->add_property('alien_share_dir' => '_share' );
 
 # alien_selection_method: name of method for selecting file: (todo: newest, manual)
 #   default is specified later, when this is undef (see alien_check_installed_version)
@@ -63,7 +63,20 @@ __PACKAGE__->add_property( alien_selection_method => 'newest' );
 # alien_build_commands: arrayref of commands for building
 __PACKAGE__->add_property( 
   alien_build_commands => 
-  default => [ '%pconfigure --prefix=%s', 'make', 'make install' ],
+  default => [ '%pconfigure --prefix=%s', 'make' ],
+);
+
+# alien_test_commands: arrayref of commands for testing the library
+# note that this might be better tacked onto the build-time commands
+__PACKAGE__->add_property( 
+  alien_test_commands => 
+  default => [ ],
+);
+
+# alien_build_commands: arrayref of commands for installing the library
+__PACKAGE__->add_property( 
+  alien_install_commands => 
+  default => [ 'make install' ],
 );
 
 # alien_version_check: command to execute to check if install/version
@@ -91,13 +104,14 @@ __PACKAGE__->add_property( 'alien_repository_class'   => {} );
 #  ConfigData  #
 ################
 
-# build_share_dir: full path to the shared directory specified in alien_share_dir
-# pkgconfig: hashref of A::B::PkgConfig objects created from .pc file found in build_share_dir
+# working_directory: full path to the extracted source or binary of the library
+# pkgconfig: hashref of A::B::PkgConfig objects created from .pc file found in working_directory
 # install_type: either system or share
 # version: version number installed or available
 # Cflags: holder for cflags if manually specified
 # Libs:   same but libs
 # name: holder for name as needed by pkg-config
+# finished_installing: set to true once ACTION_install is finished, this helps testing now and real checks later
 
 ############################
 #  Initialization Methods  #
@@ -107,43 +121,202 @@ sub new {
   my $class = shift;
   my %args = @_;
 
-  my $install_dir = ($args{alien_share_dir} ||= '_install');
-
   # merge default and user-defined repository classes
   $args{alien_repository_class}{$_} ||= $default_repository_class{$_} 
     for keys %default_repository_class;
 
-  # initialize M::B property share_dir 
-  if (! defined $args{share_dir}) {
-    # no share_dir property
-    $args{share_dir} = $install_dir;
-  } elsif (! ref $args{share_dir}) {
-    # share_dir is a scalar, meaning dist
-    $args{share_dir} = { dist => [$install_dir, $args{share_dir}] };
-  } elsif (! ref $args{share_dir}{dist}) {
-    # share_dir is like {dist => scalar}, so upconvert to arrayref
-    $args{share_dir} = { dist => [$install_dir, $args{share_dir}{dist}] };
-  } else {
-    # share_dir is like {dist => [...]}, so unshift
-    unshift @{$args{share_dir}{dist}}, $install_dir;
-  }
-
   my $self = $class->SUPER::new(%args);
 
-  # store full path to alien_share_dir, used in interpolate
-  $self->config_data( 
-    build_share_dir => File::Spec->catdir( $self->base_dir(), $install_dir )
-  );
+  # setup additional temporary directories, and yes we have to add File::ShareDir manually
+  $self->_add_prereq( 'requires', 'File::ShareDir', '1.00' );
+  $self->alien_init_temp_dir;
 
   # force newest for all automated testing 
   #TODO (this probably should be checked for "input needed" rather than blindly assigned)
   if ($ENV{AUTOMATED_TESTING}) {
     $self->alien_selection_method('newest');
-  } 
+  }
+
+  $self->config_data( 'finished_installing' => 0 );
 
   return $self;
 }
 
+sub alien_init_temp_dir {
+  my $self = shift;
+  my $temp_dir = $self->alien_temp_dir;
+  my $share_dir = $self->alien_share_dir;
+
+  # make sure we are in base_dir
+  local $CWD = $self->base_dir;
+
+  unless ( -d $temp_dir ) {
+    mkdir $temp_dir or croak "Could not create temporary directory $temp_dir";
+  }
+  $self->add_to_cleanup( $temp_dir );
+
+  unless ( -d $share_dir ) {
+    mkdir $share_dir or croak "Could not create temporary directory $share_dir";
+  }
+  $self->add_to_cleanup( $share_dir );
+
+  # add share dir to share dir list
+  my $share_dirs = $self->share_dir;
+  unshift @{ $share_dirs->{dist} }, $share_dir;
+  $self->share_dir( $share_dirs );
+  {
+    local $CWD = $share_dir;
+    open my $fh, '>', 'README' or die "Could not open README for writing (in directory $share_dir)\n";
+    print $fh <<'END';
+This README file is autogenerated by Alien::Base. 
+
+Currently it exists for testing purposes, but it might eventually contain information about the file(s) installed.
+END
+  }
+}
+
+####################
+#  ACTION methods  #
+####################
+
+sub ACTION_code {
+  my $self = shift;
+  $self->depends_on('alien') unless $self->notes('ACTION_alien_completed');
+  $self->SUPER::ACTION_code;
+}
+
+sub ACTION_alien {
+  my $self = shift;
+
+  $self->config_data( name => $self->alien_name );
+
+  my $version;
+  $version = $self->alien_check_installed_version
+    unless $Force;
+
+  if ($version) {
+    $self->config_data( install_type => 'system' );
+    $self->config_data( version => $version );
+    return;
+  }
+
+  my @repos = $self->alien_create_repositories;
+
+  my $cabinet = Alien::Base::ModuleBuild::Cabinet->new;
+
+  foreach my $repo (@repos) {
+    $cabinet->add_files( $repo->probe() );
+  }
+
+  $cabinet->sort_files;
+
+  {
+    local $CWD = $self->alien_temp_dir;
+
+    my $file = $cabinet->files->[0];
+    $version = $file->version;
+
+    print "Downloading File: " . $file->filename . " ... ";
+    my $filename = $file->get;
+    print "Done\n";
+
+    print "Extracting Archive ... ";
+    my $ae = Archive::Extract->new( archive => $filename );
+    $ae->extract;
+    print "Done\n";
+
+    my $extract_path = $ae->extract_path;
+    $self->config_data( working_directory => $extract_path );
+    $CWD = $extract_path;
+
+    print "Building library ... ";
+    unless ($self->alien_do_commands('build')) {
+      print "Failed\n";
+      croak "Build not completed";
+    }
+    print "Done\n";
+
+  }
+
+  $self->config_data( install_type => 'share' );
+
+  my $pc = $self->alien_load_pkgconfig;
+  my $pc_version = (
+    $pc->{$self->alien_name} || $pc->{_manual}
+  )->keyword('Version');
+
+  if (! $version and ! $pc_version) {
+    carp "Library looks like it installed, but no version was determined";
+    $self->config_data( version => 0 );    
+    return
+  }
+
+  if ( $version and $pc_version and versioncmp($version, $pc_version)) {
+    carp "Version information extracted from the file name and pkgconfig data disagree";
+  } 
+
+  $self->config_data( version => $pc_version || $version );
+
+  # prevent building multiple times (on M::B::dispatch)
+  $self->notes( 'ACTION_alien_completed' => 1 );
+  return;
+}
+
+sub ACTION_test {
+  my $self = shift;
+  $self->SUPER::ACTION_test;
+
+  local $CWD = $self->config_data( 'working_directory' );
+  print "Testing library (if applicable) ... ";
+  $self->alien_do_commands('test') or die "Failed\n";
+  print "Done\n";
+}
+
+sub ACTION_install {
+  my $self = shift;
+  $self->SUPER::ACTION_install;
+
+  {
+    my $target = $self->alien_library_destination;
+    local $CWD = $target;
+
+    # The only form of introspection that exists is to see that the README file
+    # which was placed in the share_dir (default _share) exists where we expect 
+    # after installation.
+    unless ( -e 'README' ) {
+      die "share_dir mismatch detected ($target)\n"
+    }
+  }
+
+  {
+    local $CWD = $self->config_data( 'working_directory' );
+    print "Installing library ... ";
+    $self->alien_do_commands('install') or die "Failed\n";
+    print "Done\n";
+  }
+
+  $self->alien_refresh_manual_pkgconfig( $self->alien_library_destination );
+  $self->config_data( 'finished_installing' => 1 );
+
+  # to refresh config_data
+  $self->SUPER::ACTION_config_data;
+  $self->SUPER::ACTION_install;
+}
+
+#######################
+#  Pre-build Methods  #
+#######################
+
+sub alien_check_installed_version {
+  my $self = shift;
+  my $command = $self->alien_version_check;
+
+  my %result = $self->do_system($command, {verbose => 0});
+  my $version = $result{stdout} || 0;
+
+  return $version;
+}
+
 sub alien_create_repositories {
   my $self = shift;
 
@@ -183,125 +356,6 @@ sub alien_create_repositories {
 
 }
 
-sub alien_init_temp_dir {
-  my $self = shift;
-  my $dir_name = $self->alien_temp_dir;
-  my $install_dir = $self->config_data('build_share_dir');
-
-  # make sure we are in base_dir
-  local $CWD = $self->base_dir;
-
-  unless ( -d $dir_name ) {
-    mkdir $dir_name or croak "Could not create temporary directory $dir_name";
-  }
-  $self->add_to_cleanup( $dir_name );
-
-  # if install_dir does not exist, create AND mark for add_to_cleanup
-  unless ( -d $install_dir ) {
-    mkdir $install_dir or croak "Could not create temporary directory $install_dir";
-    $self->add_to_cleanup( $install_dir );
-  }
-}
-
-####################
-#  ACTION methods  #
-####################
-
-sub ACTION_code {
-  my $self = shift;
-  $self->depends_on('alien') unless $self->notes('ACTION_alien_completed');
-  $self->SUPER::ACTION_code;
-}
-
-sub ACTION_alien {
-  my $self = shift;
-
-  $self->config_data( name => $self->alien_name );
-
-  my $version;
-  $version = $self->alien_check_installed_version
-    unless $Force;
-
-  if ($version) {
-    $self->config_data( install_type => 'system' );
-    $self->config_data( version => $version );
-    return;
-  }
-
-  $self->alien_init_temp_dir;
-  my @repos = $self->alien_create_repositories;
-
-  my $cabinet = Alien::Base::ModuleBuild::Cabinet->new;
-
-  foreach my $repo (@repos) {
-    $cabinet->add_files( $repo->probe() );
-  }
-
-  $cabinet->sort_files;
-
-  {
-    local $CWD = $self->alien_temp_dir;
-
-    my $file = $cabinet->files->[0];
-    $version = $file->version;
-
-    print "Downloading File: " . $file->filename . " ... ";
-    my $filename = $file->get;
-    print "Done\n";
-
-    print "Extracting Archive ... ";
-    my $ae = Archive::Extract->new( archive => $filename );
-    $ae->extract;
-    $CWD = $ae->extract_path;
-    print "Done\n";
-
-    print "Building library ... ";
-    unless ($self->alien_build) {
-      print "Failed\n";
-      croak "Build not completed";
-    }
-    print "Done\n";
-
-  }
-
-  $self->config_data( install_type => 'share' );
-
-  my $pc = $self->alien_load_pkgconfig;
-  my $pc_version = (
-    $pc->{$self->alien_name} || $pc->{_manual}
-  )->keyword('Version');
-
-  if (! $version and ! $pc_version) {
-    carp "Library looks like it installed, but no version was determined";
-    $self->config_data( version => 0 );    
-    return
-  }
-
-  if ( $version and $pc_version and versioncmp($version, $pc_version)) {
-    carp "Version information extracted from the file name and pkgconfig data disagree";
-  } 
-
-  $self->config_data( version => $pc_version || $version );
-
-  # prevent building multiple times (on M::B::dispatch)
-  $self->notes( 'ACTION_alien_completed' => 1 );
-  return;
-}
-
-#######################
-#  Pre-build Methods  #
-#######################
-
-sub alien_check_installed_version {
-  my $self = shift;
-  my $command = $self->alien_version_check;
-
-  my %result = $self->do_system($command);
-  my $version = $result{stdout} || 0;
-
-  return $version;
-}
-
 sub alien_validate_repo {
   my $self = shift;
   my ($repo) = @_;
@@ -319,41 +373,30 @@ sub alien_validate_repo {
   return $self->os_type eq $platform;
 }
 
+sub alien_library_destination {
+  my $self = shift;
+  my $lib_dir = $self->install_destination('lib');
+  my $dist_name = $self->dist_name;
+  my $dest = File::Spec->catdir( $lib_dir, qw/auto share dist/, $dist_name );
+  return $dest;
+}
+
 ###################
 #  Build Methods  #
 ###################
 
-sub alien_build {
+sub alien_do_commands {
   my $self = shift;
+  my $phase = shift;
 
-  my $commands = $self->alien_build_commands;
+  my $attr = "alien_${phase}_commands";
+  my $commands = $self->$attr();
 
   foreach my $command (@$commands) {
 
-    # hack Mac LDFLAGS
-    if (($^O eq 'darwin') and ($command =~ /\bmake(?:\.pl)?$/)) {
-      # probe for LDFLAGS variable in make database
-      my $vars = `make -p -n` || '';
-      my $ldflags = '';
-      if ($vars =~ /^\s*LDFLAGS\h*=\h*(.*)$/m) {
-        $ldflags = $1;
-        print STDERR "Found LDFLAGS = $ldflags\n";
-        $ldflags .= ' ';
-        
-      }
-
-      # add needed flag and set LDFLAGS env var
-      $ldflags .= '-headerpad_max_install_names';
-      $ENV{LDFLAGS} = $ldflags;
-      print STDERR "Using LDFLAGS = $ldflags\n";
-
-      # tell make to use the env vars over internal variables
-      $command .= ' -e';
-    }
-
     my %result = $self->do_system( $command );
     unless ($result{success}) {
-      carp "External command ($command) failed! Error: $?\n";
+      carp "External command ($result{command}) failed! Error: $?\n";
       return 0;
     }
   }
@@ -365,6 +408,9 @@ sub alien_build {
 # futher it either captures or tees depending on the value of $Verbose
 sub do_system {
   my $self = shift;
+  my $opts = ref $_[-1] ? pop : { verbose => 1 };
+
+  my $verbose = $Verbose || $opts->{verbose};
 
   # prevent build process from cwd-ing from underneath us
   local $CWD;
@@ -373,7 +419,7 @@ sub do_system {
   my @args = map { $self->alien_interpolate($_) } @_;
 
   my ($out, $err, $success) = 
-    $Verbose
+    $verbose
     ? tee     { $self->SUPER::do_system(@args) }
     : capture { $self->SUPER::do_system(@args) }
   ;
@@ -396,7 +442,7 @@ sub alien_interpolate {
   my ($string) = @_;
 
   my $prefix = $self->alien_exec_prefix;
-  my $share  = $self->config_data('build_share_dir');
+  my $share  = $self->alien_library_destination;
   my $name   = $self->alien_name;
 
   # substitute:
@@ -432,7 +478,7 @@ sub alien_exec_prefix {
 sub alien_load_pkgconfig {
   my $self = shift;
 
-  my $dir = $self->config_data('build_share_dir');
+  my $dir = $self->config_data( 'working_directory' );
   my $pc_files = $self->rscan_dir( $dir, qr/\.pc$/ );
 
   my %pc_objects = map { 
@@ -448,6 +494,19 @@ sub alien_load_pkgconfig {
   return \%pc_objects;
 }
 
+sub alien_refresh_manual_pkgconfig {
+  my $self = shift;
+  my ($dir) = @_;
+
+  my $pc_objects = $self->config_data( 'pkgconfig' );
+  $pc_objects->{_manual} = $self->alien_generate_manual_pkgconfig($dir)
+    or croak "Could not autogenerate pkgconfig information";
+  
+  $self->config_data( pkgconfig => $pc_objects );
+
+  return 1;
+}
+
 sub alien_generate_manual_pkgconfig {
   my $self = shift;
   my ($dir) = @_;
@@ -527,27 +586,6 @@ sub alien_find_lib_paths {
   return { lib => \@lib_paths, inc => \@inc_paths, so_files => \@so_files };
 }
 
-###########################
-#  Final Install Methods  #
-###########################
-
-# overload c_i_m to handle Mac's dylib relocalization
-sub copy_if_modified {
-  my $self = shift;
-  my $to_path = $self->SUPER::copy_if_modified(@_) or return;
-
-  return $to_path unless $^O eq 'darwin';
-  return $to_path unless $to_path =~ /\.dylib$/;
-
-  # handle dylib path relocalization
-  # circumvent Alien::Base::MB's interpolation engine
-  $to_path = File::Spec->rel2abs($to_path);
-  $self->SUPER::do_system("install_name_tool -id $to_path $to_path")
-    or carp("dylib localization failed");
-
-  return $to_path;
-}
-
 1;
 
 __END__
